import { User } from 'react-feather'
import styled, { createGlobalStyle } from 'styled-components'
import TextInput from '../../../molecules/TextInput'
import Button from '../../../molecules/Button'

const Variables = createGlobalStyle({
	':root': {
		'--size-topbar': '4rem',
	},
})

const Container = styled('div')({
	maxWidth: 1080,
	padding: '0 1rem',
	boxSizing: 'border-box',
	margin: '0 auto',
})

const Base = styled('div')({
	position: 'fixed',
	top: 0,
	left: 0,
	width: '100%',
	backgroundColor: 'var(--color-bg)',
	zIndex: 2,
	'~ *': {
		paddingTop: 'var(--size-topbar)',
	},
})

const Contents = styled('div')({
	display: 'flex',
	alignItems: 'center',
	justifyContent: 'space-between',
	height: 'var(--size-topbar)',
})

const Sides = styled('div')({
	width: '6rem',
})

const RightSide = styled(Sides)({
	textAlign: 'right',
})

const Search = styled('div')({
	flex: 'auto',
	display: 'flex',
	margin: '0 1rem',
})

const StyledTextInput = styled(TextInput)({
	flex: 'auto',
	marginRight: '1rem',
})

const TopBar = ({
	query = '',
}) => {
	return (
		<>
			<Variables />
			<Base>
				<Container>
					<Contents>
						<Sides>
							Bruhbot
						</Sides>
						<Search>
							<StyledTextInput

							/>
							<Button>
								Search
							</Button>
						</Search>
						<RightSide>
							<User />
						</RightSide>
					</Contents>
				</Container>
			</Base>
		</>
	)
}

export default TopBar
