import Folder from './Folder'

export default class Item {
	id?: string
	name: string
	url: string
	folder: Folder
	createdAt: Date
	updatedAt: Date
	deletedAt?: Date

	static resolve(data: Record<string, unknown>): Item {
		return {
			id: data['id'] as string,
			name: data['name'] as string,
			url: data['url'] as string,
			folder: { id: data['root_folder_id'] } as Folder,
			createdAt: new Date(data['created_at'] as string),
			updatedAt: new Date(data['updated_at'] as string),
			deletedAt: data['deleted_at'] ? new Date(data['deleted_at'] as string) : null,
		}
	}

	static formatForView(item: Item) {
		return {
			...item,
			createdAt: item.createdAt.toISOString(),
			updatedAt: item.updatedAt.toISOString(),
			deletedAt: item.deletedAt?.toISOString() || null,
		}
	}
}
